@@ -32,7 +32,7 @@
 #include "filecheck.h"
 #include "auth.h"
 
-static const gchar RCSid[]="$Id: auth.c,v 1.9 2001/01/09 23:48:13 jirka Exp $";
+static const gchar RCSid[]="$Id: auth.c,v 1.11 2001/04/24 05:03:48 jirka Exp $";
 
 
 /* Local prototypes */
@@ -65,6 +65,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     gchar *addr;
     Xauth *xa;
     guint i;
+    gchar *hostname;
 
     if (!d)
 	return FALSE;
@@ -105,6 +106,14 @@ gdm_auth_secure_display (GdmDisplay *d)
     /* Create new random cookie */
     gdm_cookie_generate (d);
 
+    hostname = g_new0 (gchar, 1024);
+
+    if (gethostname (hostname, 1023) == 0) {
+        g_free( d->hostname );
+        d->hostname = g_strdup( hostname );
+    }
+    g_free( hostname );
+
     /* Find FQDN or IP of display host */
     hentry = gethostbyname (d->hostname);
 
@@ -276,7 +285,6 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, gchar *homedir)
 
     fclose (af);
     XauUnlockAuth (d->userauth);
-    gdm_setenv ("XAUTHORITY", d->userauth);
 
     gdm_debug ("gdm_auth_user_add: Done");
 
