@@ -36,7 +36,7 @@
 #include "verify.h"
 #include "display.h"
 
-static const gchar RCSid[]="$Id: gdm.c,v 1.20 2001/02/27 21:45:11 jirka Exp $";
+static const gchar RCSid[]="$Id: gdm.c,v 1.27 2001/05/01 23:27:34 jirka Exp $";
 
 
 /* Local functions */
@@ -58,11 +58,16 @@ gboolean gdm_first_login = TRUE;
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
 gchar *GdmSessDir = NULL;
+gchar *GdmGnomeDefaultSession = NULL;
 gchar *GdmAutomaticLogin = NULL;
+gchar *GdmConfigurator = NULL;
+gboolean GdmConfigAvailable = FALSE;
+gboolean GdmSystemMenu = FALSE;
 gchar *GdmGreeter = NULL;
 gchar *GdmChooser = NULL;
 gchar *GdmLogDir = NULL;
 gchar *GdmDisplayInit = NULL;
+gchar *GdmSessionDir = NULL;
 gchar *GdmPreSession = NULL;
 gchar *GdmPostSession = NULL;
 gchar *GdmHalt = NULL;
@@ -77,6 +82,7 @@ gchar *GdmRootPath = NULL;
 gchar *GdmDefaultLocale = NULL;
 gboolean  GdmKillInitClients = FALSE;
 gint  GdmUserMaxFile = 0;
+gint  GdmSessionMaxFile = 0;
 gboolean  GdmXdmcp = FALSE;
 gint  GdmDispPerHost = 0;
 gint  GdmMaxPending = 0;
@@ -89,8 +95,11 @@ gint  GdmMaxIndirectWait = 0;
 gboolean  GdmDebug = FALSE;
 gboolean  GdmVerboseAuth = FALSE;
 gboolean  GdmAllowRoot = FALSE;
+gboolean  GdmAllowRemoteRoot = FALSE;
 gint  GdmRelaxPerms = 0;
 gint  GdmRetryDelay = 0;
+gchar *GdmTimedLogin = NULL;
+gint GdmTimedLoginDelay = 0;
 
 /* set in the main function */
 static char **stored_argv = NULL;
@@ -131,22 +140,31 @@ gdm_config_parse (void)
     GdmKillInitClients = gnome_config_get_bool (GDM_KEY_KILLIC);
     GdmLogDir= gnome_config_get_string (GDM_KEY_LOGDIR);
     GdmPidFile = gnome_config_get_string (GDM_KEY_PIDFILE);
-    GdmPostSession = gnome_config_get_string (GDM_KEY_POSTSESS);
+    GdmSessionDir = gnome_config_get_string (GDM_KEY_SESSDIR);
     GdmPreSession = gnome_config_get_string (GDM_KEY_PRESESS);
+    GdmPostSession = gnome_config_get_string (GDM_KEY_POSTSESS);
+    GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
+    GdmConfigAvailable = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
+    GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
     GdmReboot = gnome_config_get_string (GDM_KEY_REBOOT);
     GdmRetryDelay = gnome_config_get_int (GDM_KEY_RETRYDELAY);
     GdmRootPath = gnome_config_get_string (GDM_KEY_ROOTPATH);
     GdmDefaultLocale = gnome_config_get_string (GDM_KEY_LOCALE);
     GdmServAuthDir = gnome_config_get_string (GDM_KEY_SERVAUTH);
-    GdmSessDir= gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmSessDir = gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmGnomeDefaultSession = gnome_config_get_string (GDM_KEY_GNOMEDEFAULTSESSION);
     GdmUser = gnome_config_get_string (GDM_KEY_USER);
     GdmUserAuthDir = gnome_config_get_string (GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = gnome_config_get_string (GDM_KEY_UAUTHFILE);
     GdmUserAuthFB = gnome_config_get_string (GDM_KEY_UAUTHFB);
+    GdmTimedLogin = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
+    GdmTimedLoginDelay = gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
 
     GdmAllowRoot = gnome_config_get_bool (GDM_KEY_ALLOWROOT);
+    GdmAllowRemoteRoot = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEROOT);
     GdmRelaxPerms = gnome_config_get_int (GDM_KEY_RELAXPERM);
     GdmUserMaxFile = gnome_config_get_int (GDM_KEY_MAXFILE);
+    GdmSessionMaxFile = gnome_config_get_int (GDM_KEY_SESSIONMAXFILE);
     GdmVerboseAuth = gnome_config_get_bool (GDM_KEY_VERBAUTH);
 
     GdmXdmcp = gnome_config_get_bool (GDM_KEY_XDMCP);
@@ -163,13 +181,37 @@ gdm_config_parse (void)
 
     gnome_config_pop_prefix();
 
-    if (GdmAutomaticLogin != NULL &&
-	strcmp (GdmAutomaticLogin, "root") == 0) {
-	    gdm_info ("gdm_config_parse: Root cannot be autologged in, turing off");
+    /* sanitize some values */
+
+    if (gdm_string_empty (GdmAutomaticLogin)) {
 	    g_free (GdmAutomaticLogin);
 	    GdmAutomaticLogin = NULL;
     }
 
+    if (GdmAutomaticLogin != NULL &&
+	strcmp (GdmAutomaticLogin, "root") == 0) {
+	    gdm_info (_("gdm_config_parse: Root cannot be autologged in, turing off automatic login"));
+	    g_free (GdmAutomaticLogin);
+	    GdmAutomaticLogin = NULL;
+    }
+
+    if (gdm_string_empty (GdmTimedLogin)) {
+	    g_free (GdmTimedLogin);
+	    GdmTimedLogin = NULL;
+    }
+
+    if (GdmTimedLogin != NULL &&
+	strcmp (GdmTimedLogin, "root") == 0) {
+	    gdm_info (_("gdm_config_parse: Root cannot be autologged in, turing off timed login"));
+	    g_free (GdmTimedLogin);
+	    GdmTimedLogin = NULL;
+    }
+
+    if (GdmTimedLoginDelay < 10) {
+	    gdm_info (_("gdm_config_parse: TimedLoginDelay less then 10, so I will just use 10."));
+	    GdmTimedLoginDelay = 10;
+    }
+
     /* Prerequisites */ 
     if (GdmGreeter == NULL)
 	gdm_fail (_("gdm_config_parse: No greeter specified."));
@@ -181,7 +223,7 @@ gdm_config_parse (void)
 	GdmLogDir = GdmServAuthDir;
 
     if (GdmSessDir == NULL) 
-	gdm_fail (_("gdm_config_parse: No sessions directory specified."));
+	gdm_error (_("gdm_config_parse: No sessions directory specified."));
 
 
     /* Find local X server definitions */
@@ -275,6 +317,7 @@ gdm_config_parse (void)
     gdm_verify_check ();
 
     seteuid (0);
+    setegid (0);
 }
 
 
@@ -331,11 +374,17 @@ gdm_local_servers_start (GdmDisplay *d)
 {
     if (d && d->type == TYPE_LOCAL) {
 	gdm_debug ("gdm_local_servers_start: Starting %s", d->name);
-	if ( ! gdm_display_manage (d))
-		gdm_display_unmanage (d);
 
-	/* only the first local display gets autologged in */
-	gdm_first_login = FALSE;
+	/* only the first local display has timed login going on */
+	if (gdm_first_login)
+		d->timed_login_ok = TRUE;
+
+	if ( ! gdm_display_manage (d)) {
+		gdm_display_unmanage (d);
+		/* only the first local display gets autologged in */
+		gdm_first_login = FALSE;
+	}
+
     }
 }
 
@@ -391,7 +440,21 @@ gdm_cleanup_children (void)
     /* Declare the display dead */
     d->slavepid = 0;
     d->dispstat = DISPLAY_DEAD;
-	    
+
+    if ( ! GdmSystemMenu &&
+	(status == DISPLAY_REBOOT ||
+	 status == DISPLAY_HALT)) {
+	    gdm_info (_("gdm_child_action: Reboot or Halt request when there is no system menu from display %s"), d->name);
+	    status = DISPLAY_REMANAGE;
+    }
+
+    if (d->type != TYPE_LOCAL &&
+	(status == DISPLAY_REBOOT ||
+	 status == DISPLAY_HALT)) {
+	    gdm_info (_("gdm_child_action: Reboot or Halt request from a non-local display %s"), d->name);
+	    status = DISPLAY_REMANAGE;
+    }
+
     /* Autopsy */
     switch (status) {
 	
