@@ -32,6 +32,18 @@
  */
 GtkWidget *get_widget(gchar *widget_name);
 
+
+typedef struct _GdmConfigSession GdmConfigSession;
+struct _GdmConfigSession {
+   char *name;
+   char *script_contents;
+   gboolean changable; /* if we had trouble reading it it's not */
+   gboolean changed;
+   gboolean renamed;
+   char *old_name;
+   gboolean is_default;
+};
+
 /* Set a GtkEntry to 'value'. entry_name is retrieved from the GladeXML 
  * pointed to by 'GUI'. 'value' should always have been allocated (usually 
  * with gnome_config_get_string), as it is freed here.
@@ -43,8 +55,8 @@ GtkWidget *get_widget(gchar *widget_name);
                               (char *)value); \
            g_free(value); \
         }
-#define gdm_radio_set(radio_name, value) \
-        if (value >= 0) { \
+#define gdm_radio_set(radio_name, value, maximum) \
+        if (value >= 0 && value <= maximum) { \
 	   gchar *widget_name = g_strdup_printf ("%s_%d", radio_name, value); \
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get_widget(widget_name)), TRUE); \
 	   g_free (widget_name); \
@@ -67,6 +79,13 @@ GtkWidget *get_widget(gchar *widget_name);
         if (value) \
            gnome_font_picker_set_font_name(GNOME_FONT_PICKER(get_widget(font_name)), \
                                            (char *)value);
+#define gdm_color_set(picker_name, value) \
+        if (value) { \
+	   GdkColor color; \
+	   if (gdk_color_parse (value, &color)) \
+		   gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (get_widget (picker_name)), \
+					       color.red, color.green, color.blue, 0); \
+	}
 
 /* Some more macros for readable coding of the gnome_config_set_* functions */
 
@@ -85,14 +104,30 @@ GtkWidget *get_widget(gchar *widget_name);
         gnome_config_set_int(key, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget(toggle_name)))?1:0);
 
 #define gdm_icon_write(icon_name, key) \
-        if (gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(get_widget(icon_name)))) \
-           gnome_config_set_string(key, gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(get_widget(icon_name)))); \
+	{ \
+	char *icon = hack_icon_entry_get_icon(GNOME_ICON_ENTRY(get_widget(icon_name))); \
+        if (icon != NULL) \
+           gnome_config_set_string(key, icon); \
         else \
-           gnome_config_set_string(key, "");
+           gnome_config_set_string(key, ""); \
+	g_free (icon); \
+	}
 
 #define gdm_font_write(picker_name, key) \
         gnome_config_set_string(key, gnome_font_picker_get_font_name(GNOME_FONT_PICKER(get_widget(picker_name))));
 
+#define gdm_color_write(picker_name, key) \
+	{ \
+		char *value; \
+		guint8 r, g, b; \
+		gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (get_widget (picker_name)), \
+					   &r, &g, &b, NULL); \
+		value = g_strdup_printf ("#%02x%02x%02x", (int)r, (int)g, (int)b); \
+		gnome_config_set_string (key, value); \
+		g_free (value); \
+	}
+
+
 
 /* Function Prototypes */
 
@@ -133,6 +168,12 @@ void
 change_xdmcp_sensitivity               (GtkButton       *button,
                                         gpointer         user_data);
 void
+change_background_sensitivity_image    (GtkButton       *button,
+                                        gpointer         user_data);
+void
+change_background_sensitivity_color    (GtkButton       *button,
+                                        gpointer         user_data);
+void
 set_face_sensitivity                   (GtkButton       *button,
                                         gpointer         user_data);
 void handle_server_add_or_edit         (gchar           *string,
@@ -158,3 +199,32 @@ move_server_up                         (GtkButton       *button,
 void
 move_server_down                       (GtkButton       *button,
                                         gpointer         user_data);
+void
+session_text_edited (GtkEditable *text, gpointer data);
+void
+modify_session_name (GtkEntry *entry, gpointer data);
+
+
+void
+sessions_clist_row_selected                  (GtkCList *clist,
+					      gint row,
+					      gint column,
+					      GdkEventButton *event,
+					      gpointer user_data);
+void
+set_new_default_session (GtkButton *button,
+			 gpointer user_data);
+void
+add_session_real (gchar *new_session_name, gpointer data);
+void
+add_session (GtkButton *button,
+	     gpointer user_data);
+void
+remove_session (GtkButton *button,
+		gpointer user_data);
+void
+session_text_edited (GtkEditable *text, gpointer data);
+void
+modify_session_name (GtkEntry *entry, gpointer data);
+void 
+session_directory_modified (GtkEntry *entry, gpointer data);
